@@ -1,13 +1,13 @@
 #ifndef lint
-static char rcsId[]="$Header: /debian/home/gnomecvs/gnome/gtk-xmhtml/XmHTML.c,v 1.16 1998/01/07 01:45:35 unammx Exp $";
+static char rcsId[]="$Header: /debian/home/gnomecvs/gnome/gtk-xmhtml/XmHTML.c,v 1.19 1998/01/14 05:49:43 unammx Exp $";
 #endif
 /*****
 * XmHTML.c : XmHTML main routines
 *
-* This file Version	$Revision: 1.16 $
+* This file Version	$Revision: 1.19 $
 *
 * Creation date:		Thu Nov 21 05:02:44 GMT+0100 1996
-* Last modification: 	$Date: 1998/01/07 01:45:35 $
+* Last modification: 	$Date: 1998/01/14 05:49:43 $
 * By:					$Author: unammx $
 * Current State:		$State: Exp $
 *
@@ -35,6 +35,44 @@ static char rcsId[]="$Header: /debian/home/gnomecvs/gnome/gtk-xmhtml/XmHTML.c,v
 /*****
 * ChangeLog 
 * $Log: XmHTML.c,v $
+* Revision 1.19  1998/01/14 05:49:43  unammx
+* Tue Jan 13 22:04:43 1998  Federico Mena  <federico@bananoid.nuclecu.unam.mx>
+*
+* 	* gtk-xmhtml.c (gtk_xmhtml_new): The widget starts up frozen and
+*  	thaws itself when it is realized.  This fixes all of the problems
+*  	regarding realization, gc creation, and window background setting.
+*
+* (Federico and Miguel)
+*
+* Revision 1.18  1998/01/14 04:11:44  unammx
+* Tue Jan 13 22:04:43 1998  Federico Mena  <federico@bananoid.nuclecu.unam.mx>
+*
+* 	* Lots of changes all over the place to fix colors.  Things are
+* 	*almost* working right now.  I think I'm only missing setting the
+* 	window backgrounds appropriately.  Several things were done:
+*
+* 		- Motif's color and gc fields from Core and XmManager were
+* 		  replicated inside the GtkXmHTML widget structure.
+*
+* 		- Macros were created in toolkit.h to use these fields.
+*
+* 		- Instead of the old kludgy set_{fore,back}ground_internal
+* 		  functions, we now set the window background directly.
+* 		  This does not work perfectly; I'll look into it.
+*
+* 		- I created a shade_color() function in colors.c (ok, ok,
+* 		  I stole it from gtkstyle.c) which mimics XmGetColors()
+* 		  -- it calculates shaded colors for the 3D look.
+*
+* 	I hope to fix the remaining problems with window backgrounds real
+* 	soon now.
+*
+* Revision 1.17  1998/01/09 06:10:22  unammx
+* Fixed (?) background colors of the HTML widget.  I'm not 100% sure I did it
+* the right way, but it seems to work.
+*
+* - Federico
+*
 * Revision 1.16  1998/01/07 01:45:35  unammx
 * Gtk/XmHTML is ready to be used by the Gnome hackers now!
 * Weeeeeee!
@@ -2370,7 +2408,15 @@ XmHTMLTextSetString(TWidget w, String text)
 	XtVaSetValues(html->html.work_area, 
 		XmNbackground, html->html.body_bg, NULL);
 #else
-	fprintf (stderr, "SHOULD SET THE BACKGROUND COLOR\n");
+	{
+		/* FIXME: is setting the window background the same as setting
+		 * the XmNbackground resource on the work_area?
+		 */
+
+		GdkColor c;
+		c.pixel = html->html.body_bg;
+		gdk_window_set_background(html->html.work_area->window, &c);
+	}
 #endif
 	/* get new values for top, bottom & highlight */
 	_XmHTMLRecomputeColors(html);
