@@ -1,13 +1,13 @@
 #ifndef lint
-static char rcsId[]="$Header: /debian/home/gnomecvs/gnome/gtk-xmhtml/colors.c,v 1.5 1998/01/07 01:45:36 unammx Exp $";
+static char rcsId[]="$Header: /debian/home/gnomecvs/gnome/gtk-xmhtml/colors.c,v 1.9 1998/01/15 01:40:58 unammx Exp $";
 #endif
 /*****
 * colors.c : XmHTML color allocation routines
 *
-* This file Version	$Revision: 1.5 $
+* This file Version	$Revision: 1.9 $
 *
 * Creation date:		Mon Dec 16 13:57:41 GMT+0100 1996
-* Last modification: 	$Date: 1998/01/07 01:45:36 $
+* Last modification: 	$Date: 1998/01/15 01:40:58 $
 * By:					$Author: unammx $
 * Current State:		$State: Exp $
 *
@@ -35,6 +35,44 @@ static char rcsId[]="$Header: /debian/home/gnomecvs/gnome/gtk-xmhtml/colors.c,v
 /*****
 * ChangeLog 
 * $Log: colors.c,v $
+* Revision 1.9  1998/01/15 01:40:58  unammx
+* Comment fixes - Federico
+*
+* Revision 1.8  1998/01/15 01:34:03  unammx
+* Wed Jan 14 19:28:01 1998  Federico Mena  <federico@bananoid.nuclecu.unam.mx>
+*
+* 	* colors.c (my_get_colors): Now we allocate the colors, just as
+* 	Motif does.  I think they should be freed sometime, but they are
+* 	not.  Lesstif does not free them, either.  I don't know if OSF
+* 	Motif ever frees these colors...
+*
+* Revision 1.7  1998/01/14 04:11:48  unammx
+* Tue Jan 13 22:04:43 1998  Federico Mena  <federico@bananoid.nuclecu.unam.mx>
+*
+* 	* Lots of changes all over the place to fix colors.  Things are
+* 	*almost* working right now.  I think I'm only missing setting the
+* 	window backgrounds appropriately.  Several things were done:
+*
+* 		- Motif's color and gc fields from Core and XmManager were
+* 		  replicated inside the GtkXmHTML widget structure.
+*
+* 		- Macros were created in toolkit.h to use these fields.
+*
+* 		- Instead of the old kludgy set_{fore,back}ground_internal
+* 		  functions, we now set the window background directly.
+* 		  This does not work perfectly; I'll look into it.
+*
+* 		- I created a shade_color() function in colors.c (ok, ok,
+* 		  I stole it from gtkstyle.c) which mimics XmGetColors()
+* 		  -- it calculates shaded colors for the 3D look.
+*
+* 	I hope to fix the remaining problems with window backgrounds real
+* 	soon now.
+*
+* Revision 1.6  1998/01/10 02:27:02  unammx
+* First attempt at fixing the RecomputeColors functions.  They are still
+* not perfect, as scrollbar colors are affected, too. - Federico
+*
 * Revision 1.5  1998/01/07 01:45:36  unammx
 * Gtk/XmHTML is ready to be used by the Gnome hackers now!
 * Weeeeeee!
@@ -479,6 +517,256 @@ _XmHTMLFreeColors(XmHTMLWidget html)
 	*/
 }
 
+#ifndef WITH_MOTIF
+
+/* This color shading method is taken from gtkstyle.c.  Some
+ *  modifications made by me - Federico
+ */
+
+static void
+rgb_to_hls (gdouble *r, gdouble *g, gdouble *b)
+{
+	gdouble min;
+	gdouble max;
+	gdouble red;
+	gdouble green;
+	gdouble blue;
+	gdouble h, l, s;
+	gdouble delta;
+
+	red = *r;
+	green = *g;
+	blue = *b;
+
+	if (red > green) {
+		if (red > blue)
+			max = red;
+		else
+			max = blue;
+
+		if (green < blue)
+			min = green;
+		else
+			min = blue;
+	} else {
+		if (green > blue)
+			max = green;
+		else
+			max = blue;
+
+		if (red < blue)
+			min = red;
+		else
+			min = blue;
+	}
+
+	l = (max + min) / 2;
+	s = 0;
+	h = 0;
+
+	if (max != min) {
+		if (l <= 0.5)
+			s = (max - min) / (max + min);
+		else
+			s = (max - min) / (2 - max - min);
+
+		delta = max -min;
+		if (red == max)
+			h = (green - blue) / delta;
+		else if (green == max)
+			h = 2 + (blue - red) / delta;
+		else if (blue == max)
+			h = 4 + (red - green) / delta;
+
+		h *= 60;
+		if (h < 0.0)
+			h += 360;
+	}
+
+	*r = h;
+	*g = l;
+	*b = s;
+}
+
+static void
+hls_to_rgb (gdouble *h, gdouble *l, gdouble *s)
+{
+	gdouble hue;
+	gdouble lightness;
+	gdouble saturation;
+	gdouble m1, m2;
+	gdouble r, g, b;
+
+	lightness = *l;
+	saturation = *s;
+
+	if (lightness <= 0.5)
+		m2 = lightness * (1 + saturation);
+	else
+		m2 = lightness + saturation - lightness * saturation;
+	m1 = 2 * lightness - m2;
+
+	if (saturation == 0) {
+		*h = lightness;
+		*l = lightness;
+		*s = lightness;
+	} else {
+		hue = *h + 120;
+		while (hue > 360)
+			hue -= 360;
+		while (hue < 0)
+			hue += 360;
+
+		if (hue < 60)
+			r = m1 + (m2 - m1) * hue / 60;
+		else if (hue < 180)
+			r = m2;
+		else if (hue < 240)
+			r = m1 + (m2 - m1) * (240 - hue) / 60;
+		else
+			r = m1;
+
+		hue = *h;
+		while (hue > 360)
+			hue -= 360;
+		while (hue < 0)
+			hue += 360;
+
+		if (hue < 60)
+			g = m1 + (m2 - m1) * hue / 60;
+		else if (hue < 180)
+			g = m2;
+		else if (hue < 240)
+			g = m1 + (m2 - m1) * (240 - hue) / 60;
+		else
+			g = m1;
+
+		hue = *h - 120;
+		while (hue > 360)
+			hue -= 360;
+		while (hue < 0)
+			hue += 360;
+
+		if (hue < 60)
+			b = m1 + (m2 - m1) * hue / 60;
+		else if (hue < 180)
+			b = m2;
+		else if (hue < 240)
+			b = m1 + (m2 - m1) * (240 - hue) / 60;
+		else
+			b = m1;
+
+		*h = r;
+		*l = g;
+		*s = b;
+	}
+}
+
+static void
+shade_color(GdkColor *a, GdkColor *b, gdouble k)
+{
+	gdouble red;
+	gdouble green;
+	gdouble blue;
+
+	/* Special case for black, so that it looks pretty... */
+
+	if ((a->red == 0) && (a->green == 0) && (a->blue == 0)) {
+		a->red = 32768;
+		a->green = 32768;
+		a->blue = 32768;
+	}
+
+	red = (gdouble) a->red / 65535.0;
+	green = (gdouble) a->green / 65535.0;
+	blue = (gdouble) a->blue / 65535.0;
+
+	rgb_to_hls (&red, &green, &blue);
+
+	green *= k;
+
+	if (green > 1.0)
+		green = 1.0;
+	else if (green < 0.0)
+		green = 0.0;
+
+	blue *= k;
+
+	if (blue > 1.0)
+		blue = 1.0;
+	else if (blue < 0.0)
+		blue = 0.0;
+
+	hls_to_rgb (&red, &green, &blue);
+
+	b->red = (int) (red * 65535.0 + 0.5);
+	b->green = (int) (green * 65535.0 + 0.5);
+	b->blue = (int) (blue * 65535.0 + 0.5);
+}
+
+#define HIGHLIGHT_MULT 0.7
+#define LIGHT_MULT     1.5
+#define DARK_MULT      0.5
+
+static void
+my_get_colors(GdkColormap *colormap, gulong background, gulong *top, gulong *bottom, gulong *highlight)
+{
+	GdkColor cbackground;
+	GdkColor ctop, cbottom, chighlight;
+
+	/* I think this should use a ColorContext instead of allocating colors itself... - Federico */
+
+	/* FIXME: The colors that are allocated here are never freed.
+	 * I think we can save the pixel values in static variables so that we
+	 * can call gdk_colors_free() on the next invocation of the function.
+	 */
+
+	cbackground.pixel = background;
+	my_x_query_colors(colormap, &cbackground, 1);
+
+	if (top) {
+		shade_color(&cbackground, &ctop, LIGHT_MULT);
+		gdk_color_alloc(colormap, &ctop);
+		*top = ctop.pixel;
+	}
+
+	if (bottom) {
+		shade_color(&cbackground, &cbottom, DARK_MULT);
+		gdk_color_alloc(colormap, &cbottom);
+		*bottom = cbottom.pixel;
+	}
+
+	if (highlight) {
+		shade_color(&cbackground, &chighlight, HIGHLIGHT_MULT);
+		gdk_color_alloc(colormap, &chighlight);
+		*highlight = chighlight.pixel;
+	}
+}
+
+static void
+set_widget_colors(GtkXmHTML *html, gulong *top, gulong *bottom, gulong *highlight)
+{
+	GdkColor c;
+	
+	if (top) {
+		c.pixel = *top;
+		gdk_gc_set_foreground(html->top_shadow_gc, &c);
+	}
+
+	if (bottom) {
+		c.pixel = *bottom;
+		gdk_gc_set_foreground(html->bottom_shadow_gc, &c);
+	}
+
+	if (highlight) {
+		c.pixel = *highlight;
+		gdk_gc_set_foreground(html->highlight_gc, &c);
+		html->highlight_color = *highlight;
+	}
+}
+
+#endif
+
 /*****
 * Name: 		_XmHTMLRecomputeColors
 * Return Type: 	void
@@ -499,10 +787,10 @@ _XmHTMLRecomputeColors(XmHTMLWidget html)
 	*/
 	if(html->html.gc != NULL)
 	{
+#ifdef WITH_MOTIF
 		Pixel top = None, bottom = None, highlight = None;
 		Arg args[3];
 
-#ifdef WITH_MOTIF
 		XmGetColors(XtScreen((Widget)html), html->core.colormap,
 			html->html.body_bg, NULL, &top, &bottom, &highlight);
 		XtSetArg(args[0], XmNtopShadowColor, top);
@@ -510,8 +798,10 @@ _XmHTMLRecomputeColors(XmHTMLWidget html)
 		XtSetArg(args[2], XmNhighlightColor, highlight);
 		XtSetValues((Widget)html, args, 3);
 #else
-		/* FEDERICO */
-		fprintf (stderr, "%s is incomplete\n", __FUNCTION__);
+		gulong top, bottom, highlight;
+		
+		my_get_colors(gtk_widget_get_colormap(GTK_WIDGET(html)), html->html.body_bg, &top, &bottom, &highlight);
+		set_widget_colors(html, &top, &bottom, &highlight);
 #endif
 	}
 }
@@ -544,8 +834,10 @@ _XmHTMLRecomputeHighlightColor(XmHTMLWidget html, Pixel bg_color)
 		XtSetArg(args[0], XmNhighlightColor, highlight);
 		XtSetValues((Widget)html, args, 1);
 #else
-		/* FEDERICO */
-		fprintf (stderr, "%s is incomplete\n", __FUNCTION__);
+		gulong highlight;
+
+		my_get_colors(gtk_widget_get_colormap(GTK_WIDGET(html)), html->html.body_bg, NULL, NULL, &highlight);
+		set_widget_colors(html, NULL, NULL, &highlight);
 #endif
 	}
 }
