@@ -615,13 +615,14 @@ _XmHTMLFormAddTextArea(XmHTMLWidget html, String attributes, String text)
 	entry->size      = cols;
 	entry->maxlength = rows;
 
-	/* FIXME: use document colors if allowed 
-	 *  if(html->html.allow_form_coloring)
-	 *  {
-	 *      XtSetArg(args[argc], XmNbackground, html->html.body_bg); argc++;
-	 *      XtSetArg(args[argc], XmNforeground, html->html.body_fg); argc++;
-	 * }
-	 */
+	if(html->html.allow_form_coloring) {
+		/* FIXME: how to set the text area's colors? */
+#if 0
+		XtSetArg(args[argc], XmNbackground, html->html.body_bg); argc++;
+		XtSetArg(args[argc], XmNforeground, html->html.body_fg); argc++;
+#endif
+	}
+
 	textw = gtk_text_new (NULL, NULL);
  	gtk_container_add (GTK_CONTAINER (html), textw);
 	
