@@ -205,10 +205,18 @@ gtk_xmhtml_init (GtkXmHTML *html)
 	gtk_xmhtml_reset_pending_flags (html);
 
 	/* Gtk port bits */
-	html->frozen = 0;
 	html->children = NULL;
+
+	html->background_pixel = 0;
+	html->foreground_pixel = 0;
+	html->bottom_shadow_gc = NULL;
+	html->top_shadow_gc    = NULL;
+	html->highlight_gc     = NULL;
+	html->highlight_color  = 0;
 }
 
+static void gtk_xmhtml_create_widgets (GtkXmHTML *html);
+
 GtkWidget *
 gtk_xmhtml_new (void)
 {
@@ -218,7 +226,10 @@ gtk_xmhtml_new (void)
 	GTK_WIDGET(html)->allocation.width  = 200;
 	GTK_WIDGET(html)->allocation.height = 200;
 	html->initialized = 0;
+	html->frozen = 0;
+	gtk_xmhtml_create_widgets (html);
 	gtk_xmhtml_source (html, "<body></body>");
+	html->frozen = 1;
 	return GTK_WIDGET (html);
 }
 
@@ -405,7 +416,8 @@ gtk_xmhtml_get_type (void)
 			sizeof (GtkXmHTMLClass),
 			(GtkClassInitFunc) gtk_xmhtml_class_init,
 			(GtkObjectInitFunc) gtk_xmhtml_init,
-			(GtkArgFunc) NULL
+			(GtkArgSetFunc) NULL,
+			(GtkArgGetFunc) NULL,
 		};
 
 		gtk_xmhtml_type = gtk_type_unique (gtk_container_get_type (), &gtk_xmhtml_info);
@@ -602,21 +614,20 @@ CheckGC(XmHTMLWidget html)
 		GdkGCValues xgc;
 
 		xgc.function = GDK_COPY;
-		xgc.foreground = GTK_WIDGET(html)->style->fg[GTK_STATE_NORMAL];
-		xgc.background = GTK_WIDGET(html)->style->bg[GTK_STATE_NORMAL];
-		html->html.gc = gdk_gc_new_with_values (GTK_WIDGET (html)->window, &xgc,
+		xgc.foreground.pixel = Toolkit_StyleColor_Foreground(html);
+		xgc.background.pixel = Toolkit_StyleColor_Background(html);
+		html->html.gc = gdk_gc_new_with_values (GTK_WIDGET(html)->window, &xgc,
 							GDK_GC_FOREGROUND | GDK_GC_BACKGROUND |
 							GDK_GC_FUNCTION);
-		fprintf (stderr, "FIXME: missing call to XmHTMLRecomputeColors\n");
-/*		_XmHTMLRecomputeColors(html); */
+		_XmHTMLRecomputeColors(html);
 
 		_XmHTMLDebug(1, ("XmHTML.c: CheckGC, gc created\n"));
 	}
 	/* background image gc */
 	if(html->html.body_images_enabled && html->html.bg_gc == NULL)
 	{
-		html->html.bg_gc = gdk_gc_new (GTK_WIDGET (html)->window);
-		gdk_gc_copy (html->html.gc, html->html.bg_gc);
+		html->html.bg_gc = gdk_gc_new (GTK_WIDGET(html)->window);
+		gdk_gc_copy (html->html.bg_gc, html->html.gc);
 	}
 
 	_XmHTMLDebug(1, ("XmHTML.c: CheckGC End\n"));
@@ -791,24 +802,35 @@ vertical_scroll (GtkAdjustment *adj, gpointer user_data)
 	_XmHTMLMoveToPos (html->html.vsb, html, adj->value);
 }
 
-/*****
- * Name: 		CreateHTMLWidget
- * Return Type: 	void
- * Description: 	creates the HTML TWidget
- *			The actual area we use to draw into is a drawingAreaWidget.
- * In: 
- *	html:		TWidget to be created.
- * Returns:
- *	nothing
- *****/
 static void
-CreateHTMLWidget(XmHTMLWidget html)
+drawing_area_realized (GtkWidget *widget, gpointer data)
+{
+	GtkXmHTML *html = data;
+	GdkColor c;
+
+	html->background_pixel = widget->style->bg[GTK_STATE_NORMAL].pixel;
+	html->foreground_pixel = widget->style->fg[GTK_STATE_NORMAL].pixel;
+
+	html->bottom_shadow_gc = gdk_gc_new (widget->window);
+	gdk_gc_copy (html->bottom_shadow_gc, widget->style->dark_gc[GTK_STATE_NORMAL]);
+
+	html->top_shadow_gc = gdk_gc_new (widget->window);
+	gdk_gc_copy (html->top_shadow_gc, widget->style->light_gc[GTK_STATE_NORMAL]);
+
+	html->highlight_gc = gdk_gc_new (widget->window);
+	gdk_gc_copy (html->highlight_gc, widget->style->bg_gc[GTK_STATE_PRELIGHT]);
+	html->highlight_color = widget->style->bg[GTK_STATE_PRELIGHT].pixel;
+
+	gtk_xmhtml_thaw (html);
+
+	/* FIXME: remember to destroy the GCs later */
+}
+
+static void
+gtk_xmhtml_create_widgets (GtkXmHTML *html)
 {
 	GtkWidget *draw_area;
 	int events;
-	int vsb_width, hsb_height;
-
-	_XmHTMLDebug(1, ("XmHTML.c: CreateHTMLWidget Start\n"));
 
 	/* Check if user provided a work area */
 	if(html->html.work_area == NULL)
@@ -817,7 +839,10 @@ CreateHTMLWidget(XmHTMLWidget html)
 		gtk_drawing_area_size (GTK_DRAWING_AREA (draw_area), 40, 40);
 		
 		gtk_xmhtml_manage (GTK_CONTAINER (html), draw_area);
-		
+
+		gtk_signal_connect (GTK_OBJECT (draw_area), "realize",
+				    (GtkSignalFunc) drawing_area_realized, html);
+
 		gtk_signal_connect (GTK_OBJECT (draw_area), "expose_event",
 				    (GtkSignalFunc) gtk_xmhtml_expose_event, html);
 		
@@ -855,6 +880,7 @@ CreateHTMLWidget(XmHTMLWidget html)
 				       /* not yet handled */
 				       | GDK_KEY_PRESS_MASK
 				       | GDK_KEY_RELEASE_MASK);
+
 		gtk_widget_show (draw_area);
 	}
 
@@ -878,6 +904,25 @@ CreateHTMLWidget(XmHTMLWidget html)
 				    (GtkSignalFunc) vertical_scroll, html);
 	}
 
+}
+
+/*****
+ * Name: 		CreateHTMLWidget
+ * Return Type: 	void
+ * Description: 	creates the HTML TWidget
+ *			The actual area we use to draw into is a drawingAreaWidget.
+ * In: 
+ *	html:		TWidget to be created.
+ * Returns:
+ *	nothing
+ *****/
+static void
+CreateHTMLWidget(XmHTMLWidget html)
+{
+	int vsb_width, hsb_height;
+
+	_XmHTMLDebug(1, ("XmHTML.c: CreateHTMLWidget Start\n"));
+
 	/* 
 	* subtract margin_width once to minimize number of calcs in
 	* the paint routines: every thing rendered starts at an x position
@@ -902,7 +947,7 @@ gtk_xmhtml_realize (GtkWidget *widget)
 	g_return_if_fail (GTK_IS_XMHTML (widget));
 	
 	GTK_WIDGET_SET_FLAGS (widget, GTK_REALIZED);
-	
+
 	attributes.window_type = GDK_WINDOW_CHILD;
 	attributes.x = widget->allocation.x;
 	attributes.y = widget->allocation.y;
@@ -916,7 +961,7 @@ gtk_xmhtml_realize (GtkWidget *widget)
 	
 	attributes_mask = GDK_WA_X | GDK_WA_Y | GDK_WA_VISUAL | GDK_WA_COLORMAP;
 	
-	widget->window = gdk_window_new (widget->parent->window, &attributes, 
+	widget->window = gdk_window_new (gtk_widget_get_parent_window (widget), &attributes, 
 					 attributes_mask);
 	gdk_window_set_user_data (widget->window, widget);
 	
@@ -1510,12 +1555,15 @@ gtk_xmhtml_sync_reformat (GtkXmHTML *html)
 	/* reset some important vars */
 	ResetWidget(html, html->free_images_needed);
 
-#if 0
-	/* FIXME: I dont support this yet :-( */
-		/* reset background color */
-	XtVaSetValues(w_new->html.work_area, 
-		      XmNbackground, w_new->html.body_bg, NULL);
-#endif
+	{
+		/* FIXME: is setting the window background semantically equivalent
+		 * to setting the XmNbackground resource on the work_area?
+		 */
+
+		GdkColor c;
+		c.pixel = html->html.body_bg;
+		gdk_window_set_background(html->html.work_area->window, &c);
+	}
 
 	/* get new values for top, bottom & highlight */
 	_XmHTMLRecomputeColors(html);
@@ -1575,6 +1623,10 @@ gtk_xmhtml_sync_redraw (GtkXmHTML *html)
 static void
 gtk_xmhtml_sync (GtkXmHTML *html)
 {
+	if (!html->initialized){
+		html->initialized = 1;
+		XmHTML_Initialize (html, html, html->html.source);
+	}
 	if (html->parse_needed)
 		gtk_xmhtml_sync_parse (html);
 	if (html->reformat_needed)
@@ -1605,7 +1657,7 @@ gtk_xmhtml_try_sync (GtkXmHTML *html)
 void
 gtk_xmhtml_freeze (GtkXmHTML *html)
 {
-	html->frozen = 1;
+	html->frozen++;
 }
 
 void
@@ -1614,8 +1666,8 @@ gtk_xmhtml_thaw (GtkXmHTML *html)
 	if (!html->frozen)
 		return;
 
-	html->frozen = 0;
-	gtk_xmhtml_sync (html);
+	html->frozen--;
+	gtk_xmhtml_try_sync (html);
 }
 
 void
@@ -1623,10 +1675,6 @@ gtk_xmhtml_source (GtkXmHTML *html, char *html_source)
 {
 	int parse = FALSE;
 
-	if (!html->initialized){
-		html->initialized = 1;
-		XmHTML_Initialize (html, html, html_source);
-	}
 	/* If we already have some HTML source code */
 	if (html->html.source){
 		if (html_source){	/* new text supplied */
@@ -1650,7 +1698,36 @@ gtk_xmhtml_source (GtkXmHTML *html, char *html_source)
 	}
 	html->html.value = html->html.source;
 	html->parse_needed = parse;
-	gtk_xmhtml_try_sync (html);
+	
+	if (html->initialized)
+		gtk_xmhtml_try_sync (html);
+}
+
+/* XXX: This function does an XQueryColors() the hard way, because there is
+ * no corresponding function in Gdk.
+ */
+
+void
+my_x_query_colors(GdkColormap *colormap,
+		  GdkColor    *colors,
+		  gint         ncolors)
+{
+	XColor *xcolors;
+	gint    i;
+
+	xcolors = g_new(XColor, ncolors);
+	for (i = 0; i < ncolors; i++)
+		xcolors[i].pixel = colors[i].pixel;
+
+	XQueryColors(gdk_display, GDK_COLORMAP_XCOLORMAP(colormap), xcolors, ncolors);
+
+	for (i = 0; i < ncolors; i++) {
+		colors[i].red   = xcolors[i].red;
+		colors[i].green = xcolors[i].green;
+		colors[i].blue  = xcolors[i].blue;
+	}
+
+	g_free(xcolors);
 }
 
 void
