@@ -1,3 +1,5 @@
+(use-modules (toolkits gtk))
+
 (define call/cc call-with-current-continuation)
 
 (define-macro (with-return . forms)
@@ -77,8 +79,7 @@
 			 (if (gtk-widget-visible? b)
 			     (gtk-widget-hide b)
 			     (gtk-widget-show b))
-			 (if break-iteration (break-iteration #f))
-			 ))
+			 (if break-iteration (break-iteration #f))))
 
 	(table (gtk-table-new 3 3 #f))
 	(buttons (map (lambda (bt)
@@ -98,8 +99,8 @@
 	     (clicked (list-ref buttons (cadddr bt))))
 	 (gtk-signal-connect b "clicked" 
 			     (lambda () (toggle-button clicked)))
-	 (gtk-signal-connect b "enter_notify_event"
-			     (lambda (ev) (pk 'enter ev) #f))
+	 ;; (gtk-signal-connect b "enter_notify_event"
+	 ;;		     (lambda (ev) (pk 'enter ev) #f))
 	 (gtk-table-attach table b x (+ 1 x) y (+ 1 y)
 			   '(expand fill) '(expand fill)
 			   0 0)
@@ -136,7 +137,7 @@
   (let ((hbox (gtk-hbox-new #f 5))
 	(label (gtk-label-new "Hello World")))
     (define (make-frame title init-label)
-      (let ((frame (gtk-frame-new "Frame 1"))
+      (let ((frame (gtk-frame-new title))
 	    (box (gtk-vbox-new #f 5))
 	    (button (gtk-button-new-with-label "switch")))
 	(gtk-box-pack-start hbox frame #t #t 0)
@@ -287,9 +288,9 @@
     
     (set! window (gtk-color-selection-dialog-new
 		  "color selection dialog"))
-    (let ((colorsel (gtk-color-selection-dialog-colorsel window)))
-      (gtk-signal-connect colorsel "color_changed" 
-			  (lambda () (pk 'changed))))
+    ;; (let ((colorsel (gtk-color-selection-dialog-colorsel window)))
+    ;; (gtk-signal-connect colorsel "color_changed" 
+    ;;		  (lambda () (pk 'changed))))
     (gtk-signal-connect
      (gtk-color-selection-dialog-ok-button window)
      "clicked" (lambda () (pk 'ok)))
@@ -556,4 +557,6 @@
 
 (gtk-rc-parse "testgtkrc")
 (create-main-window)
-(gtk-main)
+
+(define (main args)
+  (gtk-main))
