@@ -32,11 +32,6 @@
   return self;
 }
 
-- add:(Gtk_Widget *) child
-{
-  return [self pack_start_defaults:child];
-}
-
 - pack_end_defaults:(Gtk_Widget *) child
 {
   gtk_box_pack_end_defaults(gtkbox, child->gtkwidget);
