@@ -10,12 +10,15 @@
   GtkEntry *gtkentry;
 }
 - castGtkEntry:(GtkEntry *) castitem;
-- init;
+- initWithMaxLength:(guint16) max_length;
 - set_text:(gchar *) text;
 - append_text:(gchar *) text;
 - prepend_text:(gchar *) text;
 - set_position:(gint) position;
 - (gchar *) get_text;
+- set_visibility:(BOOL) visible;
+- set_editable:(BOOL) editable;
+- select_region:(gint) start :(gint) end;
 @end
 
 #endif /* OBGTK_ENTRY_H */
