@@ -7,6 +7,11 @@
   return [super castGtkWidget:GTK_WIDGET(castitem)];
 }
 
+- initWithMaxLength:(guint16) max_length
+{
+  return [self castGtkEntry:GTK_ENTRY(gtk_entry_new_with_max_length(max_length))];
+}
+
 - init
 {
   return [self castGtkEntry:GTK_ENTRY(gtk_entry_new())];
@@ -40,4 +45,22 @@
 {
   return gtk_entry_get_text(gtkentry);
 }
+
+- select_region:(gint) start :(gint) end
+{
+  gtk_entry_select_region(gtkentry, start, end);
+  return self;
+}
+
+- set_visibility:(BOOL) visible
+{
+  gtk_entry_set_visibility(gtkentry, visible);
+  return self;
+}
+
+- set_editable:(BOOL) editable
+{
+  gtk_entry_set_editable(gtkentry, editable);
+  return self;
+}
 @end
