@@ -14,13 +14,6 @@
   return self;
 }
 
-- add:(Gtk_Widget *)widget
-{
-  GList *alist = g_list_append(NULL, (gpointer) widget->gtkwidget);
-  [self append_items:alist];
-  return self;
-}
-
 - append_items:(GList *) items
 {
   gtk_list_append_items(gtklist, items);
