@@ -10,11 +10,6 @@
 @public
   GtkMenuBar *gtkmenubar;
 }
-- init;
-- append:(Gtk_Widget *) child;
-- prepend:(Gtk_Widget *) child;
-- insert:(Gtk_Widget *) child
-    aPos:(gint) position;
 @end
 
 #endif /* OBGTK_MENU_BAR_H */
