@@ -3,6 +3,9 @@
 
 #include <obgtk/obgtkContainer.h>
 #include <obgtk/obgtkWidget.h>
+#include <obgtk/obgtkMenuItem.h>
+
+@class Gtk_MenuItem;
 
 @interface Gtk_MenuShell : Gtk_Container
 {
@@ -10,10 +13,10 @@
   GtkMenuShell *gtkmenushell;
 }
 - castGtkMenuShell:(GtkMenuShell *) castitem;
-- append:(Gtk_Widget *) child;
-- prepend:(Gtk_Widget *) child;
-- insert:(Gtk_Widget *) child
-    aPos:(gint) position;
+- appendMenuItem:(Gtk_MenuItem *) child;
+- prependMenuItem:(Gtk_MenuItem *) child;
+- insertMenuItem:(Gtk_MenuItem *) child
+	    aPos:(gint) position;
 - deactivate;
 @end
 
