@@ -7,20 +7,20 @@
   return [super castGtkContainer:GTK_CONTAINER(castitem)];
 }
 
-- append:(Gtk_Widget *) child
+- appendMenuItem:(Gtk_MenuItem *) child
 {
   gtk_menu_shell_append(gtkmenushell, child->gtkwidget);
   return self;
 }
 
-- prepend:(Gtk_Widget *) child
+- prependMenuItem:(Gtk_MenuItem *) child
 {
   gtk_menu_shell_prepend(gtkmenushell, child->gtkwidget);
   return self;
 }
 
-- insert:(Gtk_Widget *) child
-    aPos:(gint) position
+- insertMenuItem:(Gtk_MenuItem *) child
+	    aPos:(gint) position
 {
   gtk_menu_shell_insert(gtkmenushell, child->gtkwidget, position);
   return self;
