@@ -12,6 +12,12 @@ obgtk_signal_relay(GtkObject *object,
 		   gint nparams,
 		   GtkArg *args);
 
+/* You can pass this in as a GtkSignalFunc and have the user_data be
+   the name of the method to call... However it's not perfect */
+gint
+obgtk_callback_relay(GtkObject *object,
+		     gpointer data);
+
 @interface Gtk_Object : Object
 {
 @public
