@@ -67,6 +67,56 @@ obgtk_signal_relay(GtkObject *object,
   return retval;
 }
 
+gint
+obgtk_callback_relay(GtkObject *object,
+		     gpointer data)
+{
+  id anobj = nil, sigobj = nil;
+  gint retval;
+  char *selname;
+  IMP callfunc;
+  SEL callsel;
+  g_return_if_fail(object != NULL);
+  g_return_if_fail(data != NULL);
+
+  sigobj = gtk_object_get_data(object, "objc_id");
+  selname = g_malloc(strlen(data) + 30);
+
+  anobj = gtk_object_get_data(object, "obgtk_callback_relay");
+#ifdef DEBUG
+  printf("We got data %#x for %s\n", anobj, selname);
+#endif
+
+  if(!anobj)
+    anobj = sigobj;
+
+#ifdef DEBUG
+  printf("Call to method %s on %#x\n", data, anobj);
+#endif
+
+  if(!anobj)
+    {
+      printf("We don't even have anobj for callback %s\n", data);
+      return;
+    }
+
+  sprintf(selname, "%s:", (char *)data);
+  callsel = sel_get_uid(selname);
+  if(!callsel) {
+    g_free(selname);
+    objc_error(anobj, 99, "Couldn't find method %s on object.", data);
+    return;
+  }
+  callfunc = objc_msg_lookup(anobj, callsel);
+
+  if(callfunc)
+    retval = (gint)callfunc(anobj, callsel, sigobj);
+  else
+    objc_error(anobj, 99, "Couldn't find method %s on object.", data);
+  g_free(selname);
+  return retval;
+}
+
 @implementation Gtk_Object
 - init
 {
