@@ -16,7 +16,7 @@
 
 - free
 {
-  gtk_tooltips_destroy(gtktooltips);
+  gtk_tooltips_unref(gtktooltips);
   return [super free];
 }
 
