@@ -19,12 +19,16 @@
 #ifndef _CALC_H_
 #define _CALC_H_
 
+#include "structs.h"
+
 typedef enum {INFIX_NOTATION,POSTFIX_NOTATION,PREFIX_NOTATION} notation_t;
 typedef enum {	NO_ERROR=0,
-		PARSE_ERROR
+		PARSE_ERROR,
+		INTERNAL_MPW_ERROR,
+		NUMERICAL_MPW_ERROR
 	} error_t;
 
-typedef struct {
+typedef struct _calcstate_t {
 	/*about incoming stuff*/
 	notation_t notation_in; /*notation of input*/
 	int float_prec;        /*precision of floats to use*/
@@ -36,6 +40,25 @@ typedef struct {
 	int scientific_notation; /*allways scientific notation*/
 } calcstate_t;
 
+
+/*so we can use and set the yyparse function for parse errors*/
+int yyparse(void);
+
+/*spit out ordinary primitives, such as plus, minus etc*/
+char * str_normalprim(char *s,char *p, tree_t *l, tree_t *r);
+
+/*appends to S the primitive op and it's arguments*/
+char * str_prim(char *s, int op, tree_t *l, tree_t *r, tree_t *sr);
+
+/*make a string representation of an expression*/
+char * makeexprstr(char *s,tree_t *n);
+
+/*add the right parenthesis and brackets to the end of the expression*/
+char * addparenth(char *s);
+
+/*this is the function to be mstly called outsied of calc.c
+  evaluate the xpression string and give back a string with the
+  result*/
 char * evalexp(char * str, calcstate_t state,void (*errorfunc)(char *));
 
 #endif
