@@ -73,8 +73,8 @@
 {
   char jobstr[20], printqueueopt[80];
 
-  snprintf(jobstr, 20, "%d", jobid);
-  snprintf(printqueueopt, 80, "-P%s", [printer getQueueName]);
+  g_snprintf(jobstr, 20, "%d", jobid);
+  g_snprintf(printqueueopt, 80, "-P%s", [printer getQueueName]);
   return !doRunCmd(LPRM_PATH, LPRM_PATH, printqueueopt, jobstr, NULL);
 }
 @end
@@ -201,7 +201,7 @@
   numjobs = njoballoc = 0;
   jobs = NULL;
 
-  snprintf(execstr, sizeof(execstr), "%s -P%s", LPQ_PATH, queuename);
+  g_snprintf(execstr, sizeof(execstr), "%s -P%s", LPQ_PATH, queuename);
 
   lpqout = popen(execstr, "r");
 
@@ -269,8 +269,8 @@
   FILE *pout;
   char execstr[512];
   char *t;
-  snprintf(execstr, sizeof(execstr), "%s status %s",
-	   LPC_PATH, queuename);
+  g_snprintf(execstr, sizeof(execstr), "%s status %s",
+	     LPC_PATH, queuename);
   pout = popen(execstr, "r");
   if(!pout)
     return FALSE;
@@ -288,8 +288,8 @@
   FILE *pout;
   char execstr[512];
   char *t;
-  snprintf(execstr, sizeof(execstr), "%s status %s",
-	   LPC_PATH, queuename);
+  g_snprintf(execstr, sizeof(execstr), "%s status %s",
+	     LPC_PATH, queuename);
   pout = popen(execstr, "r");
   if(!pout)
     return FALSE;
@@ -310,8 +310,8 @@
   FILE *pout;
   char execstr[512];
   char *t;
-  snprintf(execstr, sizeof(execstr), "%s status %s",
-	   LPC_PATH, queuename);
+  g_snprintf(execstr, sizeof(execstr), "%s status %s",
+	     LPC_PATH, queuename);
   pout = popen(execstr, "r");
   if(!pout)
     return FALSE;
