@@ -52,23 +52,23 @@ static void *
 get_dl_func(void *handle, char *name)
 {
 	void *func;
-	char *error;
-	char *funcname = name;
+	const char *error;
+	char *funcname;
 
 #       ifdef __FreeBSD__
         funcname = g_copy_strings("_", name, NULL);
+#       else
+        funcname = g_strdup(name);
 #       endif
 
-	func = dlsym(handle, funcname);
+	func = dlsym(handle, (const char *)funcname);
 	if ((error = dlerror()) != NULL) {
 		fprintf(stderr, "get_dl_func: %s (func is %s)\n", error,
 			funcname);
 		g_assert(error == NULL);
 	}
 
-#       ifdef __FreeBSD__
 	g_free(funcname);
-#       endif
 
 	return func;
 }
@@ -78,7 +78,7 @@ static void
 init_applet_file(char *filename)
 {
 	void          *handle;
-	char          *error;
+	const char    *error;
 	char          *id;
 	AppletCmdFunc  cmd_func;
 	AppletCommand  cmd;
