@@ -57,11 +57,12 @@ enum {
 static gint
 batmon_timeout_callback (gpointer *data)
 {
-	gboolean	linestat;
+	gboolean	linestat = TRUE;
 	gint		batflag;
 	gint		batpct;
 	gint		batmin;
 	gchar		str[30];
+	gchar		*fname;
 	
 /* In an effort to be platform independent, the section of code below
    (up to the #endif) is the only Linux-specific code in this function
@@ -81,7 +82,6 @@ batmon_timeout_callback (gpointer *data)
 	gchar		*string[9];
 	gchar		*mempoint;
 	gchar		*apmstr;
-	gchar		*fname;
 	gint		i;
 	int             intval;
 	
@@ -136,6 +136,11 @@ batmon_timeout_callback (gpointer *data)
 	gtk_label_set (GTK_LABEL (bioslabel), string[1]);
 	
 	g_free (mempoint);
+#else
+	batflag = NOEXIST_MASK;
+	batpct = 100;
+	batmin = 99999999; /* some random number */
+	linestat = 1;
 #endif /* __linux__ */
 
 	/* I've never done this before; is there a better way to
@@ -331,7 +336,6 @@ static void
 init_module(void)
 {
 	tooltips = gtk_tooltips_new();
-
 	ac_pixmap_filename = gnome_unconditional_pixmap_file ("batmon-ac.xpm");
 	bat_pixmap_filename = gnome_unconditional_pixmap_file ("batmon-bat.xpm");
 
@@ -405,6 +409,9 @@ applet_cmd_func (AppletCommand *cmd)
 		case APPLET_CMD_GET_INSTANCE_PARAMS:
 			return g_strdup("");
 
+		case APPLET_CMD_ORIENTATION_CHANGE_NOTIFY:
+			break;
+
 		case APPLET_CMD_PROPERTIES:
 			fprintf(stderr, "Battery monitor properties not yet implemented\n");
 			break;
