@@ -391,8 +391,10 @@ get_interface_path(void)
   gchar *path = NULL;
 
   g_slist_foreach(interface_list, add_devpath_component, &new_devpath);
-  path = g_strdup(new_devpath->str); /* caller frees */
-  g_string_free(new_devpath, 1);
+  if (new_devpath){
+	  path = g_strdup(new_devpath->str); /* caller frees */
+	  g_string_free(new_devpath, 1);
+  }
   return path;
 }
 
@@ -461,6 +463,9 @@ applet_cmd_func(AppletCommand *cmd)
 		case APPLET_CMD_GET_INSTANCE_PARAMS:
 			return g_strdup(gtk_object_get_user_data(GTK_OBJECT(cmd->applet)));
 
+		case APPLET_CMD_ORIENTATION_CHANGE_NOTIFY:
+			break;
+
 		case APPLET_CMD_PROPERTIES:
 			properties(cmd->applet);
 			break;
