@@ -6,6 +6,9 @@
 
 BEGIN_GNOME_DECLS
 
+#define DEFAULT_APPLET_WIDTH 50
+#define DEFAULT_APPLET_HEIGHT 50
+
 
 typedef enum {
 	PANEL_POS_TOP,
@@ -27,14 +30,25 @@ typedef enum {
 
 typedef struct {
 	GtkWidget  *window;
+	GtkWidget  *table;
+	GtkWidget  *hidebutton_l_h;
+	GtkWidget  *hidebutton_r_h;
+	GtkWidget  *hidebutton_l_v;
+	GtkWidget  *hidebutton_r_v;
 	GtkWidget  *fixed;
 	PanelPos    pos;
 	PanelState  state;
 	PanelMode   mode;
-	int         step_size;
-	int         delay;
+	gint        step_size;
+	gint        delay;
+	gint        tooltips_enabled;
 	gint        enter_notify_id;
 	gint        leave_notify_id;
+	gint        visibility_notify_id;
+	gint        button_press_id;
+	gint        leave_notify_timer_tag;
+	gint        minimize_delay;
+	gint        minimized_size;
 	GtkWidget  *applet_being_dragged;
 	gint        applet_drag_click_x;
 	gint        applet_drag_click_y;
@@ -57,6 +71,8 @@ int panel_session_save (gpointer client_data,
 			GnomeInteractStyle interact_style,
 			int is_fast);
 
+void panel_reconfigure(Panel *newconfig);
+
 
 END_GNOME_DECLS
 
