@@ -1,4 +1,5 @@
 
+#include <stdlib.h>
 #include "logical_board.h"
 
 static int debug=0;
@@ -448,7 +449,7 @@ int int_abs (int i)
 	{
 	    if ((color[ x ][ y ] & D_frozen) != 0)
 	    {
-		if (rand() % 10 > 8)
+		if (random () % 10 > 8)
 		{
 		    color[ x ][ y ] &= ~D_frozen;
 		    [self update_pos : x : y];
@@ -458,7 +459,7 @@ int int_abs (int i)
     }
 
 
-    if (rand () & 1)
+    if (random () & 4)
     {
 	/* pits will be there this turn */
 
@@ -635,6 +636,10 @@ int int_abs (int i)
 {
   char *message = NULL;
 
+  if (debug)
+    printf ("logical board -- mouse button press %d, %d, %d, %d\n",
+	    x, y, n, owners_color);
+
   if (owners_color != whos_turn && owners_color != C_both)
     {
       return "cannot act on mouse click: wrong player color.";
@@ -729,15 +734,31 @@ int int_abs (int i)
       break;
     }
 
+  if (debug)
+    printf ("              -- after switch\n");
+	    
 
   if (moves == 3 && csi == csi_no_selection)
     {
+      if (debug)
+	printf ("              -- switching turns\n");
+
       [self switch_players_turn];
     }
 
+  if (debug)
+    printf ("              -- update moves left\n");
+
   [self update_moves_left];
+
+  if (debug)
+    printf ("              -- check for kings\n");
+
   [self check_for_kings];
 
+  if (debug)
+    printf ("              -- returning\n");
+
   return message;
 }
 
@@ -944,9 +965,9 @@ int int_abs (int i)
 
     do
     {
-	nx = rand () % 15;
-	ny = rand () % 11;
-	nd = rand () % 8;
+	nx = random () % 15;
+	ny = random () % 11;
+	nd = random () % 8;
     } while (board[ nx ][ ny ] != 0 &&  /* blank */
 	     board[ nx ][ ny ] != 12 && /* center pit */
 	     board[ nx ][ ny ] != 13);   /* pit */
@@ -985,8 +1006,8 @@ int int_abs (int i)
 
     do
     {
-	nx = rand () % 15;
-	ny = rand () % 11;
+	nx = random () % 15;
+	ny = random () % 11;
     } while (board[ nx ][ ny ] != 0 &&  /* blank */
 	     board[ nx ][ ny ] != 12 && /* center pit */
 	     board[ nx ][ ny ] != 13);   /* pit */
@@ -1205,7 +1226,7 @@ int int_abs (int i)
 	    y = [self y_dir_add : y : ldir];
 	    break;
 	case Lrndm:
-	    ldir = rand() % 8;
+	    ldir = random () % 8;
 	    [self beam_out : x : y : ldir];
 	    x = [self x_dir_add : x : ldir];
 	    y = [self y_dir_add : y : ldir];
@@ -1228,6 +1249,9 @@ int int_abs (int i)
 {
   if (red_bv == NULL)
     {
+      if (debug)
+	printf ("logical_board -- request_color -- giving red\n");
+
       red_bv = bv;
       [bv take_color : C_red];
       [self update_moves_left];
@@ -1236,11 +1260,17 @@ int int_abs (int i)
 
   if (green_bv == NULL)
     {
+      if (debug)
+	printf ("logical_board -- request_color -- giving green\n");
+
       green_bv = bv;
       [bv take_color : C_green];
       [self update_moves_left];
       return;
     }
+
+  if (debug)
+    printf ("logical_board -- request_color -- giving neither\n");
 }
 
 @end
